from django.db import models
from pubsub import channels
from .querysets import NotificationQuerySet


class Notification(models.Model):
    user = models.ForeignKey(
        'auth.User',
        related_name='notifications',
        on_delete=models.CASCADE
    )

    text = models.TextField()
    summary = models.CharField(max_length=255)
    kind = models.CharField(max_length=20, db_index=True)
    sent = models.DateTimeField(auto_now_add=True)
    read = models.BooleanField(default=False, db_index=True)

    objects = NotificationQuerySet.as_manager()

    def __str__(self):  # pragma: no cover
        return self.summary

    def send(self):
        channels.publish(
            'user.notifications',
            'create',
            self.kind,
            username=self.user.username,
            notification={
                'text': '' + self.text,
                'summary': '' + self.summary,
                'kind': self.kind
            },
            badge=self.user.notifications.unread().count()
        )

        channels.publish(
            'user.messages',
            'create',
            self.kind,
            username=self.user.username,
            kind=self.kind,
            text=self.summary
        )

    class Meta:
        ordering = ('-sent',)
        get_latest_by = 'sent'


class Action(models.Model):
    notification = models.ForeignKey(
        Notification,
        related_name='actions',
        on_delete=models.CASCADE
    )

    kind = models.CharField(max_length=20, db_index=True)
    text = models.CharField(max_length=100)
    url = models.URLField(max_length=512)
    ordering = models.PositiveIntegerField(default=0)

    def __str__(self):  # pragma: no cover
        return self.text

    class Meta:
        ordering = ('ordering',)
        unique_together = ('url', 'notification')



from django.conf import settings
from django.contrib.auth.models import User
from django.core.mail import EmailMultiAlternatives
from django.template.loader import render_to_string
from .models import Notification
import cron


@cron.interval(minutes=5)
def send_notifications():
    notifications_per_user = {}
    for notification in Notification.objects.filter(read=False):
        if notification.user.pk not in notifications_per_user:
            notifications_per_user[notification.user.pk] = []

        notifications_per_user[notification.user.pk].append(notification)

    for user in User.objects.filter(
        pk__in=notifications_per_user.keys()
    ).exclude(
        email=None
    ).exclude(
        email=''
    ):
        messages = []
        subject = ''

        for notification in notifications_per_user[user.pk]:
            messages.append(notification)
            notification.read = True
            notification.save(update_fields=('read',))

            if not subject:
                subject = notification.summary

        if any(messages):
            html = render_to_string(
                'email/notifications/notification.html',
                {
                    'user': user,
                    'messages': messages
                }
            )

            msg = EmailMultiAlternatives(
                subject,
                'You have notifications from Podiant',
                settings.DEFAULT_FROM_EMAIL,
                [user.email]
            )

            msg.attach_alternative(html, 'text/html')
            msg.send()



from django.views.generic.list import ListView
from django.views.generic.detail import DetailView
from pubsub import channels
from .models import Notification


class NotificationListView(ListView):
    model = Notification

    def get_queryset(self):
        return super().get_queryset().filter(
            user=self.request.user
        )

    def get(self, request, *args, **kwargs):
        from django.http.response import HttpResponseBadRequest

        action = request.GET.get('action')
        if action == 'delete':
            self.get_queryset().delete()
        elif action == 'read':
            self.get_queryset().update(read=True)
        elif action:
            return HttpResponseBadRequest('Unknown or unspecified action')

        if action:
            channels.publish(
                'user.notifications',
                'update',
                username=request.user.username,
                badge=request.user.notifications.unread().count()
            )

        return super().get(request, *args, **kwargs)


class NotificationDetailView(DetailView):
    model = Notification

    def get_queryset(self):
        return super().get_queryset().filter(
            user=self.request.user
        )

    def get(self, request, *args, **kwargs):
        from django.http.response import HttpResponseBadRequest

        action = request.GET.get('action')
        self.object = self.get_object()

        if action == 'delete':
            self.object.delete()
        elif action == 'read':
            self.object.read = True
            self.object.save(update_fields=('read',))
        else:
            return HttpResponseBadRequest('Unknown or unspecified action')

        channels.publish(
            'user.notifications',
            'update',
            username=request.user.username,
            badge=request.user.notifications.unread().count()
        )

        return self.render_to_response(
            self.get_context_data(
                object=self.object
            )
        )
